%% RBDO: Rubble-mound Breakwater

%% 1 - INITIALIZE UQLAB
%
clearvars
rng(1,'twister')
uqlab

%% 2 - COMPUTATIONAL MODEL

%% 2.1 Cost function
%
RBDOOpts.Cost.mFile = 'uq_Ex02_RubBreakwat_cost';

%% 2.2 Limit state function
%
ModelOpts.mFile = 'uq_Ex02_RubBreakwat_constraint';
myModel = uq_createModel(ModelOpts);
RBDOOpts.LimitState.Model = myModel;

%%
% Soft constraints
RBDOOpts.SoftConstraints.mFile = 'uq_Ex02_RubBreakwat_SoftCon';

%% 3 - PROBABILISTIC INPUT MODEL
%
%% 3.1 Design variables
%
RBDOOpts.Input.DesVar(1).Name = 'a';  % Armor-layer height
RBDOOpts.Input.DesVar(1).Type = 'Gaussian';
RBDOOpts.Input.DesVar(1).CoV = 0.1;

RBDOOpts.Input.DesVar(2).Name = 'b';  % Underlayer height
RBDOOpts.Input.DesVar(2).Type = 'Gaussian';
RBDOOpts.Input.DesVar(2).CoV = 0.1;

RBDOOpts.Input.DesVar(3).Name = 'c';  % Core height
RBDOOpts.Input.DesVar(3).Type = 'Gaussian';
RBDOOpts.Input.DesVar(3).CoV = 0.083;

RBDOOpts.Input.DesVar(4).Name = 'Cw';  % Crest width
RBDOOpts.Input.DesVar(4).Type = 'Gaussian';
RBDOOpts.Input.DesVar(4).CoV = 0.045;

RBDOOpts.Input.DesVar(5).Name = 'hc'; % Seaward freeboard
RBDOOpts.Input.DesVar(5).Type = 'constant';

RBDOOpts.Input.DesVar(6).Name = 'hd'; % Water depth at toe
RBDOOpts.Input.DesVar(6).Type = 'constant';

RBDOOpts.Input.DesVar(7).Name = 't'; % Core thickness
RBDOOpts.Input.DesVar(7).Type = 'constant';

RBDOOpts.Input.DesVar(8).Name = 'r'; % Core offset
RBDOOpts.Input.DesVar(8).Type = 'constant';

RBDOOpts.Input.DesVar(9).Name = 'Bt'; % Toe berm width
RBDOOpts.Input.DesVar(9).Type = 'constant';

RBDOOpts.Input.DesVar(10).Name = 'al';  % Leeward slope
RBDOOpts.Input.DesVar(10).Type = 'Gaussian';
RBDOOpts.Input.DesVar(10).CoV = 0.163;

RBDOOpts.Input.DesVar(11).Name = 'as';  % Seaward slope
RBDOOpts.Input.DesVar(11).Type = 'Gaussian';
RBDOOpts.Input.DesVar(11).CoV = 0.163;

RBDOOpts.Input.DesVar(12).Name = 'at'; % Toe berm slope
RBDOOpts.Input.DesVar(12).Type = 'Gaussian';
RBDOOpts.Input.DesVar(12).CoV = 0.163;

%% 3.2 Environmental variables
%
InputOpts.Marginals(1).Name = 'Hs' ;  % Significant wave height
InputOpts.Marginals(1).Type = 'Gaussian';
InputOpts.Marginals(1).Moments = [2.5 0.11];

InputOpts.Marginals(2).Name = 'T';  % Wave period
InputOpts.Marginals(2).Type = 'Gaussian';
InputOpts.Marginals(2).Moments = [5 1.5];

%%
myInput = uq_createInput(InputOpts);

%%
RBDOOpts.Input.EnvVar = myInput;

%% 4 - OPTIMIZATION SETUP
%
RBDOOpts.Optim.Bounds = [1 0.5 5 3 3 1 0.5 1 3 0.4 0.4 0.4; ... 
    3 1.5 7 6 5 4 3 6 6 0.59 0.59 0.59];

%% Starting Points
%
RBDOOpts.Optim.StartingPoint = [2 1 6 4.5 4 2.5 1.75 3.5 4.5 0.5 0.5 0.5];

%% Target Failure Probability
% 
RBDOOpts.TargetPf = [0.01 0.05];

%% 5 - RELIABILITY-BASED DESIGN OPTIMIZATION (RBDO)
%
RBDOOpts.Type = 'RBDO';
RBDOOpts.Method = 'two-level';

%% 5.1 Monte Carlo simulation (MCS) with CCMAES
%
MCCOpts = RBDOOpts;

%%
% Set the maximum sample size for the MCS:
MCCOpts.Reliability.Simulation.MaxSampleSize = 5e4;

%%
% Set the maximum number of iterations:
MCCOpts.Optim.MaxIter = 350;

%%
% Run the RBDO analysis:
myRBDO_MCC = uq_createAnalysis(MCCOpts);

%%
% Print out a report of the results:
uq_print(myRBDO_MCC)

%%
% Display a graphical representation of the results:
uq_display(myRBDO_MCC)

%% 5.2 Quantile Monte Carlo (QMC) with IP
%
QIPOpts = RBDOOpts;
QIP.Method = 'QMC';
QIPOpts.Optim.Method = 'IP';

%%
% Set the maximum number of iterations:
QIPOpts.Optim.MaxIter = 100;

%%
% Set the finite difference step size:
QIPOpts.Optim.IP.FDStepSize = 0.1;

%%
% Set the maximum sample size:
QIPOpts.Reliability.Simulation.MaxSampleSize = 1e4;

%%
% Run the RBDO analysis:
myRBDO_QIP = uq_createAnalysis(QIPOpts);

%%
% Print out a report of the results:
uq_print(myRBDO_QIP)

%%
% Display a graphical representation of the results:
uq_display(myRBDO_QIP)

%% 5.3 Inverse FORM with SQP
%
IFSOpts = RBDOOpts;
IFSOpts.Reliability.Method = 'IFORM';
IFSOpts.Optim.Method = 'SQP';

%%
% Set the maximum number of iterations:
IFSOpts.Optim.MaxIter = 100;

%%
% Set the finite difference step size:
IFSOpts.Optim.SQP.FDStepSize = 0.1;

%%
% Run the RBDO analysis:
myRBDO_IFS = uq_createAnalysis(IFSOpts);

%%
% Print out a report of the results:
uq_print(myRBDO_IFS)

%%
% Display a graphical representation of the results:
uq_display(myRBDO_IFS)
